#pragma once
#include <cstdint>

#include "rte_ethdev.h"
#include "rte_ether.h"

#define MAX_NUMA_NODES (64)

struct device_conf {
	int portid;
	uint16_t tx_ring_sz;
	uint16_t rx_ring_sz;
	cpuset_t core_affinity;
	int mtu;
	uint64_t rx_offloads;
	uint64_t tx_offloads;
	uint64_t rss_hf;
	
	rte_tx_callback_fn tx_fn;
	void * tx_user;

	rte_rx_callback_fn rx_fn;
	void * rx_user;

	bool timesync;
};

struct mem_conf {
	int num_elements;
	int cache_size;
	int data_room_size;
	int priv_size;
	unsigned int max_pools;
};

constexpr static uint16_t MIN_RANDOM_PORT = 1000;
constexpr static uint16_t DEFAULT_RAT_PORT = 1234;
constexpr static unsigned int INIT_DELAY = 3;
constexpr static unsigned int MAX_NODES = 64;

void
dpdk_init(struct device_conf *dconf, struct mem_conf *mconf);

void
dpdk_cleanup(struct device_conf *dconf);

struct rte_mempool *
mempool_get(int nodeid);

struct port_conf {
	const char * driver_name;
	uint64_t rxoffload;
	uint64_t txoffload;
	uint64_t rss_hf;
	bool timesync;
};

int
portconf_get(int portid, struct port_conf * out);


// constexpr static int LATENCY_MEASURE_TIMES = 10000;

// static inline void
// sync_port_clock(uint16_t portid)
//{
//    int64_t lat = 0;
//    int64_t get_time_lat;
//    int64_t write_time_lat;
//    struct timespec dum;
//    struct timespec start;
//    struct timespec end;
//
//    // measure clock_gettime latency
//    for(int i = 0; i < LATENCY_MEASURE_TIMES; i++) {
//        // end - start ~= 2x clock_gettime's latency
//        clock_gettime(CLOCK_REALTIME, &start);
//        clock_gettime(CLOCK_REALTIME, &dum);
//        clock_gettime(CLOCK_REALTIME, &end);
//
//        if (end.tv_sec != start.tv_sec) {
//            rte_exit(EXIT_FAILURE, "clock_gettime too slow\n");
//        }
//
//        // shouldn't overflow
//        lat += (end.tv_nsec - start.tv_nsec) / 2;
//    }
//    get_time_lat = lat / LATENCY_MEASURE_TIMES;
//
//    // measure rte_eth_timesync_write_time latency
//    lat = 0;
//    for(int i = 0; i < LATENCY_MEASURE_TIMES; i++) {
//        // end - start ~= rte_eth_timesync latency + clock_gettime's latency
//        clock_gettime(CLOCK_REALTIME, &dum);
//        clock_gettime(CLOCK_REALTIME, &start);
//        if (rte_eth_timesync_write_time(portid, &dum) != 0) {
//            rte_exit(EXIT_FAILURE, "failed to write time\n");
//        }
//        clock_gettime(CLOCK_REALTIME, &end);
//
//        if (end.tv_sec != start.tv_sec) {
//            rte_exit(EXIT_FAILURE, "clock_gettime too slow!\n");
//        }
//
//        // shouldn't overflow
//        int64_t elat = (end.tv_nsec - start.tv_nsec) - get_time_lat;
//        if (elat < 0) {
//            rte_exit(EXIT_FAILURE, "something is wrong with lat \n");
//        }
//        lat += elat;
//    }
//    write_time_lat = lat / LATENCY_MEASURE_TIMES;
//
//    int64_t delta = (get_time_lat + write_time_lat) / 2;
//    int64_t s2ns = (int64_t)S2NS;
//    // sync the clock
//    while (true) {
//        clock_gettime(CLOCK_REALTIME, &dum);
//        dum.tv_nsec += delta;
//        if (dum.tv_nsec > s2ns) {
//            // try again if overflow
//            continue;
//        }
//        if (rte_eth_timesync_write_time(portid, &dum) != 0) {
//            rte_exit(EXIT_FAILURE, "failed to write time\n");
//        }
//        break;
//    }
//    rte_eth_timesync_enable(portid);
//
//    printf("Sync-ed time: get lat %ld write lat %ld\n", get_time_lat,
//    write_time_lat);
//}
