#pragma once

#include "storage/drivers/driver.hh"
#include "spdk/bdev.h"
#include "spdk/bdev_zone.h"
#include "spdk/thread.h"

class birb_bdev_driver : public birb_driver
{
public:
    birb_bdev_driver(const char * dev_name);
    ~birb_bdev_driver() override;
    size_t get_capacity() override;
    birb_driver_status get_status() override;
    struct spdk_bdev * get_bdev();
    struct spdk_bdev_desc * get_bdev_desc();
    birb_driver_type get_type() override;
    size_t get_align() override;

private:
    DISALLOW_EVIL_CONSTRUCTORS(birb_bdev_driver);
    struct spdk_bdev_desc * bdev_desc;
    struct spdk_bdev * bdev;
    size_t block_sz;
    size_t block_num;
    birb_driver_status status;

    static void print_all_bdev();
    static void bdev_event_cb(enum spdk_bdev_event_type type, struct spdk_bdev * bdev,
	                void * event_ctx);
};


class birb_bdev_thread_context : public birb_driver_thread_context
{
public:
    birb_bdev_thread_context(birb_bdev_driver * driver);
    ~birb_bdev_thread_context() override;
    int read(size_t offset, size_t size, char * buffer, callback callback, void * context) override;
    int write(size_t offset, size_t size, char * buffer, callback callback, void * context) override;
    void poll() override;
    birb_driver::birb_driver_status get_status() override;

private:
    struct cb_context {
        callback cb;
        void * ctx;
    };

    DISALLOW_EVIL_CONSTRUCTORS(birb_bdev_thread_context);
    spdk_io_channel * io_channel;
    birb_driver::birb_driver_status status;
    birb_bdev_driver * driver;

    static void io_callback(struct spdk_bdev_io *bdev_io, bool success, void *cb_arg);
};
