#pragma once

#include "storage/drivers/driver.hh"
#include "spdk/nvme.h"
#include "spdk/thread.h"

class birb_nvme_driver : public birb_driver
{
public:
    birb_nvme_driver(const char * dev_name);
    ~birb_nvme_driver() override;
    size_t get_capacity() override;
    birb_driver_status get_status() override;
    birb_driver_type get_type() override;
    size_t get_align() override;

    spdk_nvme_ctrlr * get_ctrlr();
    spdk_nvme_ns * get_ns();
    spdk_nvme_io_qpair_opts * get_io_qpair_opts();

private:
    struct attach_context {
        spdk_nvme_ctrlr ** ctrlr;
        spdk_nvme_ns ** ns;
        const char * dev_name;
        int valid;
    };

    DISALLOW_EVIL_CONSTRUCTORS(birb_nvme_driver);
    birb_driver_status status;
    spdk_nvme_ctrlr * ctrlr;
    spdk_nvme_ns * ns;
    spdk_nvme_io_qpair_opts opts;

    static bool probe_cb(void *cb_ctx, const struct spdk_nvme_transport_id *trid, struct spdk_nvme_ctrlr_opts *opts);
    static void attach_cb(void *cb_ctx, const struct spdk_nvme_transport_id *trid,
	                        struct spdk_nvme_ctrlr *ctrlr, const struct spdk_nvme_ctrlr_opts *opts);
};


class birb_nvme_thread_context : public birb_driver_thread_context
{
public:
    birb_nvme_thread_context(birb_nvme_driver * driver);
    ~birb_nvme_thread_context() override;
    int read(size_t offset, size_t size, char * buffer, callback callback, void * context) override;
    int write(size_t offset, size_t size, char * buffer, callback callback, void * context) override;
    void poll() override;
    birb_driver::birb_driver_status get_status() override;

private:
    struct cb_context {
        callback cb;
        void * ctx;
    };

    DISALLOW_EVIL_CONSTRUCTORS(birb_nvme_thread_context);
    birb_driver::birb_driver_status status;
    birb_nvme_driver * driver;
    struct spdk_nvme_qpair * qpair;

    static void io_callback(void *arg, const struct spdk_nvme_cpl *completion);
    static uint32_t size_to_lba(size_t size, int lba_size);
    static uint64_t addr_to_lba(size_t addr, int lba_size);
};
