#pragma once

#include "defs.hh"

#include "spdk/thread.h"
#include <cstdlib>

class birb_driver
{
private:
    DISALLOW_EVIL_CONSTRUCTORS(birb_driver);

public:
    enum birb_driver_status{
        BIRB_SUCCESS,
        BIRB_FAIL
    };
    enum birb_driver_type{
        BIRB_DRV_NVME,
        BIRB_DRV_BDEV
    };
    virtual size_t get_capacity() = 0;
    virtual birb_driver_status get_status() = 0;
    virtual size_t get_align() = 0;
    virtual birb_driver_type get_type() = 0;
    virtual ~birb_driver() = default;
protected:
    birb_driver() = default;
};


class birb_driver_thread_context
{
private:
    DISALLOW_EVIL_CONSTRUCTORS(birb_driver_thread_context);
    
public:
    using callback = void (*)(bool, void *);
    virtual int read(size_t offset, size_t size, char * buffer, callback callback, void * context) = 0;
    virtual int write(size_t offset, size_t size, char * buffer, callback callback, void * context) = 0;
    virtual void poll() = 0;
    virtual birb_driver::birb_driver_status get_status() = 0;
    virtual ~birb_driver_thread_context() = default;
protected:
    birb_driver_thread_context() = default;
};

