#pragma once
#include <sys/endian.h>
#include <sys/types.h>
#include "defs.hh"
#include "gen.hh"
#include <random>

enum io_generator_opcode {
    IOGEN_READ,
    IOGEN_WRITE
};

enum io_generator_address_mode {
    IOGEN_ADDR_MONOTONIC_INCREASING,
    IOGEN_ADDR_UNIFORM_RANDOM
};

struct io_generator_ctx {
    unsigned long size;
    uint64_t offset;
    io_generator_opcode op;
};

//
// cur_offset is aligned to req_size
//
class io_generator {
public:
    int issue(struct io_generator_ctx * ctx, char * buf);
    io_generator(unsigned long req_size,
                    unsigned long capacity,
                    unsigned int read_pct,
                    io_generator_address_mode addr_mode);
    io_generator() = delete;
    
private:
    unsigned long cur_offset;

    const unsigned long capacity;
    const unsigned long req_size;
    const unsigned int read_pct;
    const io_generator_address_mode addr_mode;

    std::random_device rd;
    std::mt19937 rng;
    std::uniform_int_distribution<int> dist;

    std::random_device addr_rd;
    std::mt19937 addr_rng;
    std::uniform_int_distribution<uint64_t> addr_dist;
    
    DISALLOW_EVIL_CONSTRUCTORS(io_generator);
};
