import subprocess as sp
import time
import select
import os
import pwd
import sys
import datetime
import random
import re
from threading import Thread 

tc_logfile = None
	
def log_print(info):
	print(info)
	if tc_logfile != None:
		tc_logfile.write(info + "\n")
		tc_logfile.flush()

tc_output_dir=""
tc_cur_test = ""
tc_test_id = 0

def init(odir = "./results.d/"):
	global tc_output_dir
	tc_output_dir = odir
	tc_output_dir = os.path.expanduser(tc_output_dir)
	os.system("mkdir -p " + tc_output_dir)
	global tc_logfile
	tc_logfile = open(tc_output_dir + "/log.txt", "w+")

def begin(name):
	global tc_test_id
	global tc_cur_test
	tc_cur_test = name
	tc_test_id += 1
	os.system("mkdir -p " + get_odir())
	log_print("\n===== Test #" + str(tc_test_id) + " - " + tc_cur_test + " started =====")

def end():
	global tc_cur_test
	log_print("\n===== Test #" + str(tc_test_id) + " - " + tc_cur_test + " completed =====")
	tc_cur_test = ""

def get_odir():
	return tc_output_dir + "/" + tc_cur_test

SCHED_QUEUE = 1
SCHED_CPU = 2
SCHED_BEST = 4
SCHED_FEAT_WS = 1
def make_sched_flag(sched, args, feat = 0, fargs = 0):
	return (sched & 0xFF) | (args & 0xFF) << 8 | (feat & 0xFF) << 16 | (fargs & 0xFF) << 24

TUNE_RTSHARE = 2
TUNE_TFREQ = 1
def make_tune_flag(obj, val):
	return (obj & 0xFFFF) | (val & 0xFFFF) << 16 

def get_username():
    return pwd.getpwuid( os.getuid() )[0]

ssh_param = ""
def set_ssh_param(para):
	global ssh_param
	ssh_param = para

def get_ssh_param():
	global ssh_param
	return ssh_param

ssh_user = None
def set_ssh_user(user):
	global ssh_user
	ssh_user = user

def get_ssh_user():
	global ssh_user
	return ssh_user

def remote_exec(srv : list[str], cmd : str, blocking=True, check=True) -> sp.Popen:
	sub = []
	for s in srv:
		p = sp.Popen(["ssh " + ssh_param + " " + ((ssh_user + "@") if ssh_user != None else "") + s + " \"" + cmd +"\""], shell=True, stdout=sp.PIPE, stderr=sp.PIPE)
		sub.append(p)
	
	if blocking:
		for p in sub:
			p.wait()
			if check and p.returncode != 0:
				raise Exception("Command failed " + cmd)

	return sub


def check_stderr(p, sel, exclude = []):# -> tuple[bool, list[str]]:
	max_stderr_rd = 10
	err = []
	while sel.poll(1) and max_stderr_rd > 0:
		err.append(p.stderr.readline().decode().strip())
		max_stderr_rd = max_stderr_rd - 1


	good = True
	for e in err:
		e = e.strip()
		if len(e) == 0:
			continue
		
		good = False
		for exc in exclude:
			if exc in e:
				good = True
				break
			
	return good, err

# stderr threads
errthr_objs = []
errthr_sigstop = False
errthr_failed = False

def errthr_get_failed():
	return errthr_failed

def thr_check_stderr(p : sp.Popen, name: str, exclude):
	global errthr_failed
	sel = select.poll()
	sel.register(p.stderr, select.POLLIN)
	local_failed = False
	while(not errthr_sigstop):
		if (not local_failed):
			status, err = check_stderr(p, sel, exclude=exclude)
			if not status:
				errthr_failed = True
				local_failed = True
				log_print("Error detected in \"" + name + "\":")
				for e in err:
					log_print("        \"" + e + "\"")
				log_print("")
		time.sleep(random.uniform(0.001, 0.1))

def errthr_start():
	global errthr_sigstop
	global errthr_failed
	errthr_sigstop = False
	errthr_failed = False
	for thr in errthr_objs:
		thr.daemon = True
		thr.start()

def errthr_create(cp, name, exclude = None):
	global errthr_objs
	for i in range(len(cp)):
		errthr_objs.append(Thread(target = thr_check_stderr, args=(cp[i], name[i], exclude)))

def errthr_stop():
	global errthr_objs
	global errthr_sigstop
	errthr_sigstop = True
	for thr in errthr_objs:
		thr.join()
	errthr_objs.clear()

def parse_hostfile(fp):
	ret = {}
	fh = open(fp, "r")
	content = fh.readlines()
	fh.close()
	content = [x.strip() for x in content]
	for line in content:
		spl = line.split(" ")
		if len(spl) >= 2:
			ret[spl[0]] = spl[1]
			log_print("Parsed: hostname \"" + spl[0] + "\" -> \"" + spl[1] + "\"")
	return ret

def process_hostnames(names, hosts):
	ret = []
	for line in names:
		if line in hosts:
			ret.append(hosts[line])
		else:
			ret.append(line)
	return ret

def get_cpuset_core(threads):
	ret = "cpuset -l 0-" + str(threads * 2 - 1) + " "
	return ret